@@ -36,10 +36,10 @@
         if (methods[method]) {
             return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
         }
-        if (typeof method === 'object' || !method) {
+        if (typeof method === "object" || !method) {
             return methods.init.apply(this, arguments);
         }
-        $.error('Method ' + method + ' does not exist on jQuery.inplaceeditform');
+        $.error("Method " + method + " does not exist on jQuery.inplaceeditform");
     };
     $.inplaceeditform = {
         opts: {
@@ -50,7 +50,9 @@
             "disableClick": true,
             "autoSave": false,
             "unsavedChanges": "You have unsaved changes!",  // This is the only opts that can not be overwritten
-            "enableClass": "enable"
+            "enableClass": "enable",
+            "fieldTypes": "input, select, textarea",
+            "focusWhenEditing": true
         },
         configSelector: "inplaceeditform",
         formSelector: "form.inplaceeditform",
@@ -85,10 +87,10 @@
             this.each(function () {
                 var configTag = $(this).find(self.configSelector);
                 var config = configTag.attr();
-                if (self.methods.getOptBool(config, self.opts, 'disableClick')) {
+                if (self.methods.getOptBool(config, self.opts, "disableClick")) {
                     $(this).click(self.methods.disableClickCallBack);
                 }
-                $(this).bind(self.methods.getOpt(config, self.opts, 'eventInplaceEdit'), self.methods.eventInplaceEditCallBack);
+                $(this).bind(self.methods.getOpt(config, self.opts, "eventInplaceEdit"), self.methods.eventInplaceEditCallBack);
             });
             window.onbeforeunload = self.methods.onBeforeUnloadEvent;
             return {
@@ -97,7 +99,7 @@
                     self.inplaceeditfields.each(function () {
                         var configTag = $(this).find(self.configSelector);
                         var config = configTag.attr();
-                        var enableClass = self.methods.getOpt(config, self.opts, 'enableClass');
+                        var enableClass = self.methods.getOpt(config, self.opts, "enableClass");
                         $(this).addClass(enableClass);
                     });
                 },
@@ -106,7 +108,7 @@
                     self.inplaceeditfields.each(function () {
                         var configTag = $(this).find(self.configSelector);
                         var config = configTag.attr();
-                        var enableClass = self.methods.getOpt(config, self.opts, 'enableClass');
+                        var enableClass = self.methods.getOpt(config, self.opts, "enableClass");
                         $(this).removeClass(enableClass);
                     });
                 }
@@ -121,7 +123,8 @@
         },
         autoSaveCallBack: function () {
             var self = $.inplaceeditform;
-            var newValue = this.tag.val();
+            var form = this.tag.parents(self.formSelector);
+            var newValue = self.methods.getValue(form);
             if (newValue !== this.oldValue) {
                 this.tag.parents(self.formSelector).find(".apply").click();
             } else {
@@ -158,10 +161,10 @@
             data += "&__widget_height=" + $(this).innerHeight() + "px" + "&__widget_width=" + $(this).innerWidth() + "px";
             $.ajax({
                 data: data,
-                url: self.methods.getOpt(config, self.opts, 'getFieldUrl'),
+                url: self.methods.getOpt(config, self.opts, "getFieldUrl"),
                 type: "GET",
                 async: true,
-                dataType: 'json',
+                dataType: "json",
                 error: self.methods.bind(self.methods.treatmentStatusError, {"context": $(this)}),
                 success: self.methods.bind(self.methods.inplaceGetFieldSuccess, {"that": this})
             });
@@ -205,6 +208,18 @@
             }
             return val;
         },
+        getValue: function(form) {
+            var applyButton = form.find(".apply");
+            var getValue = applyButton.data("getValue"); // A hook
+            var newValue;
+            var field_id = form.find("span.field_id").html();
+            if (getValue) {
+                newValue = getValue(form, field_id);
+            } else {
+                newValue = form.find("#" + field_id).val();
+            }
+            return newValue;
+        },
         inplaceApply: function () {
             var self = $.inplaceeditform;
             var form = $(this).parents(self.formSelector);
@@ -220,14 +235,7 @@
             var configTag = form.prev().find(self.configSelector);
             var config = configTag.attr();
             var data = self.methods.getDataToRequest(configTag);
-            var field_id = form.find("span.field_id").html();
-            var getValue = $(this).data("getValue"); // A hook
-            var value;
-            if (getValue) {
-                value = getValue(form, field_id);
-            } else {
-                value = form.find("#" + field_id).val();
-            }
+            var value = self.methods.getValue(form);
             data += "&value=" + encodeURIComponent($.toJSON(value));
             var csrfmiddlewaretoken = self.methods.getCSFRToken();
             if (csrfmiddlewaretoken) {
@@ -235,10 +243,10 @@
             }
             form.data("ajaxTime", true);
             $.ajax({data: data,
-                    url: self.methods.getOpt(config, self.opts, 'saveURL'),
+                    url: self.methods.getOpt(config, self.opts, "saveURL"),
                     type: "POST",
                     async: true,
-                    dataType: 'text',
+                    dataType: "text",
                     error: self.methods.bind(self.methods.treatmentStatusError, {"context": $(this)}),
                     success: self.methods.bind(self.methods.inplaceApplySuccess, {
                         "context": $(this),
@@ -267,17 +275,10 @@
             if (csrfmiddlewaretoken) {
                 data.csrfmiddlewaretoken = csrfmiddlewaretoken;
             }
-            var field_id = form.find("span.field_id").html();
-            var getValue = $(this).data("getValue"); // A hook
-            var value;
-            if (getValue) {
-                value = getValue(form, field_id);
-            } else {
-                value = form.find("#" + field_id).val();
-            }
+            var value = self.methods.getValue(form);
             data.value = encodeURIComponent($.toJSON(value));
             form.data("ajaxTime", true);
-            form.ajaxSubmit({url: self.methods.getOpt(config, self.opts, 'saveURL'),
+            form.ajaxSubmit({url: self.methods.getOpt(config, self.opts, "saveURL"),
                              data: data,
                              async: true,
                              type: "POST",
@@ -321,9 +322,12 @@
                 alert("The server is down");
             } else if (response.errors) {
                 if (!self.methods.getOptBool(config, self.opts, "autoSave") || !parseInt(config.can_auto_save)) {
-                    form.find(".apply, .applyFile, .cancel").visible();
+                    form.animate({opacity: 1}, function () {
+                        form.find(".apply, .applyFile, .cancel").visible();
+                    });
+                } else {
+                    form.animate({opacity: 1});
                 }
-                form.animate({opacity: 1});
                 form.prepend("<ul class='errors'><li>" + response.errors + "</li></ul>");
             } else {
                 that.parent().fadeOut();
@@ -349,7 +353,7 @@
             var self = $.inplaceeditform;
             var configTag = inplace_span.find(self.configSelector);
             var config = configTag.attr();
-            var successText = self.methods.getOpt(config, self.opts, 'successText');
+            var successText = self.methods.getOpt(config, self.opts, "successText");
             if (successText) {
                 var success_message = $("<ul class='success'><li>" + successText + "</li></ul>");
                 inplace_span.prepend(success_message);
@@ -371,7 +375,7 @@
             $(this).parent().prev().fadeIn();
             var cancelFinish = $(this).data("cancelFinish");
             if (cancelFinish) {
-                cancelFinish();
+                cancelFinish(this);
             }
             $(this).parents(self.formSelector).remove();
             return false;
@@ -440,13 +444,16 @@
                     applyFileButton.click(self.methods.inplaceApplyUpload);
                     $(that).next(self.formSelector).submit(self.methods.bind(self.methods.inplaceApplyUpload, applyFileButton));
                 }
-                var fieldTag = $(that).next(self.formSelector).find("input, select, textarea");
-                fieldTag.focus();
+                var form = $(that).next(self.formSelector);
+                var fieldTag = form.find(self.methods.getOpt(config, self.opts, "fieldTypes"));
+                if (self.methods.getOptBool(config, self.opts, "focusWhenEditing")) {
+                    fieldTag.focus();
+                }
                 if (self.methods.getOptBool(config, self.opts, "autoSave") && parseInt(config.can_auto_save)) {
                     applyButton.hide();
                     cancelButton.hide();
                     applyFileButton.hide();
-                    var value = fieldTag.val();
+                    var value = self.methods.getValue(form);
                     fieldTag.blur(
                         self.methods.bind(self.methods.autoSaveCallBack, {"oldValue": value,
                                                                           "tag": fieldTag})
@@ -463,7 +470,7 @@
             var self = $.inplaceeditform;
             var fileref;
             if (media.tagName === "SCRIPT") { //if filename is a external JavaScript file
-                fileref = document.createElement('script');
+                fileref = document.createElement("script");
                 fileref.setAttribute("type", "text/javascript");
                 if (media.src !== null && media.src !== "") {
                     fileref.setAttribute("src", media.src);
